#include "animefilter.h"

#include <mythtv/mythcontext.h>




AnimeFilter::AnimeFilter(QPtrList<AnimeNode> animeList, QString oldCategory, bool oldOnlyUnwatched, MythMainWindow *parent, QString windowName,
                                    QString themeFilename, const char *name)
           : MythThemedDialog(parent, windowName, themeFilename, name) {
    EMPTY="<All Categories>";


    if (oldCategory == "") {
        oldCategory = EMPTY;
    }
    AnimeFilter::category = oldCategory;
    AnimeFilter::onlyUnwatched = oldOnlyUnwatched;


    QStringList categories = QStringList();
    for (AnimeNode* node = animeList.first(); node; node = animeList.next()) {
        QStringList currentCategories = node->getCategories();
        for (QStringList::Iterator it = currentCategories.begin(); it != currentCategories.end(); ++it) {
            QString category =  *it;
            if (categories.findIndex(category) == -1) {
                categories.append(category);
            }
        }
    }


    initTheme();


    AnimeFilter::root = new GenericTree();
    root->setInt(1);
    categories.sort();
    categories.insert(categories.begin(), "<All Categories>");
    for (QStringList::Iterator it = categories.begin(); it != categories.end(); ++it) {
        QString category =  *it;
        if (category == oldCategory) {
            root->addNode(category, 2, true);
        } else {
            root->addNode(category, 0, true);
        }
    }

    AnimeFilter::categoryList->assignTreeData(root);
    QValueList<int> path = QValueList<int>();
    path += 1;
    path += 2;
    AnimeFilter::categoryList->tryToSetActive(path);
}


AnimeFilter::~AnimeFilter() {
	delete AnimeFilter::root;
}


void AnimeFilter::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Anime", e, actions);

    for (unsigned int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];

        handled = true;
        if (action == "UP") {
            AnimeFilter::categoryList->moveUp();
        } else if (action == "DOWN") {
            AnimeFilter::categoryList->moveDown();
        } else if (action == "LEFT") {
            AnimeFilter::categoryList->popUp();
        } else if (action == "RIGHT") {
            AnimeFilter::categoryList->pushDown();
        } else if (action == "PAGEUP") {
            AnimeFilter::categoryList->pageUp();
        } else if (action == "PAGEDOWN") {
            AnimeFilter::categoryList->pageDown();
        } else if (action == "SELECT") {
            AnimeFilter::categoryList->select();
        } else if (action == "1") {
            AnimeFilter::onlyUnwatchedBox->toggle();
        } else {
            handled = false;
            VERBOSE(VB_IMPORTANT, action);
        }
    }

    if (!handled) {
        MythThemedDialog::keyPressEvent(e);
    }
}

void AnimeFilter::initTheme() {
    AnimeFilter::categoryList = getUIManagedTreeListType("categories");
    if (!AnimeFilter::categoryList) {
        VERBOSE(VB_IMPORTANT, "Cannot find managedtreelist categories");
    }

 	AnimeFilter::categoryList->showWholeTree(true);
 	AnimeFilter::categoryList->colorSelectables(true);

    connect(AnimeFilter::categoryList, SIGNAL(nodeSelected(int, IntVector*)), this, SLOT(handleTreeListSelection(int, IntVector*)));
    connect(AnimeFilter::categoryList, SIGNAL(nodeEntered(int, IntVector*)), this, SLOT(handleTreeListEntry(int, IntVector*)));




    AnimeFilter::onlyUnwatchedBox = getUICheckBoxType("onlywatched");
    if (!AnimeFilter::onlyUnwatchedBox) {
        VERBOSE(VB_IMPORTANT, "Cannot find checkbox onlywatched");
    }
    onlyUnwatchedBox->setState(AnimeFilter::onlyUnwatched);
}


void AnimeFilter::handleTreeListSelection(int nodeInt, IntVector*) {
    done(1);
}

void AnimeFilter::handleTreeListEntry(int nodeInt, IntVector*) {
    QString category = AnimeFilter::categoryList->getCurrentNode()->getString();
    if (category == "<All Categories>") {
        category = "";
    }
    AnimeFilter::category = category;
}


