#include "animenode.h"

#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>

#include <mythtv/mythcontext.h>


AnimeNode::AnimeNode(QString path, QString name)
    : GenericTree(name, 1, true) {
    AnimeNode::anidbPath = path+"/"+name+"/anidb/";
    AnimeNode::name = name;

    //TODO init everything with 0 or throw errors if something is not initializeds

	readAniDBFile();
	addEpisodes(path);
	readSeenFile();

	//VERBOSE(VB_IMPORTANT, "+a");
}

AnimeNode::~AnimeNode() {
	//VERBOSE(VB_IMPORTANT, "-a");
}


void AnimeNode::readAniDBFile() {
	QString anidbFile = AnimeNode::anidbPath+AnimeNode::name+".txt";

    QFile file( anidbFile );
    if ( file.open( IO_ReadOnly ) ) {
        QTextStream stream( &file );
        QString line;

        while ( !stream.atEnd() ) {
            line = stream.readLine(); // line of text excluding '\n'
            if (line.at(0) == '#') {
                continue;
            }

            int index = line.find('=');
            QString key = line.left(index);
            QString right = line.right(line.length()-index-1);

            right.replace("\\\\", "\\");

            right.replace("\\:", ":");
            right.replace("\\!", ":");
            right.replace("\\:", ":");
            right.replace("\\t", "\t");
            right.replace("\\f", "\f");
            right.replace("\\n", "\n");
            right.replace("\\r", "\r");
            right.replace("\\#", "#");
            right.replace("\\=", "=");
            right.replace("\\ ", " ");

            int unicodeIndex;
            while ((unicodeIndex = right.find("\\u")) != -1) {
                QString unicode = right.mid(unicodeIndex, 6);
                QString unicodeParsable = unicode.right(4);
                right.replace(unicode, QChar((uint)unicodeParsable.toInt(0, 16)));
            }
            QString value = right;

            if (key == "description") {
                AnimeNode::description = value;
            } else if (key == "name") {
                AnimeNode::name = value;
            } else if (key == "englishName") {
                AnimeNode::englishName = value;
            } else if (key == "romanjiName") {
                AnimeNode::romanjiName = value;
            } else if (key == "year") {
                //Split the year, set startYear + endYear
                bool ok;
                AnimeNode::startYear = value.left(4).toInt(&ok, 10);
                if (!ok) {
                    AnimeNode::startYear = 0;
                }

                AnimeNode::endYear = value.right(4).toInt(&ok, 10);
                if (!ok) {
                    AnimeNode::endYear = 0;
                }
            } else if (key == "rating") {
                AnimeNode::rating = value.toInt(0, 10);
            } else if (key == "voteCount") {
                AnimeNode::voteCount = value.toInt(0, 10);
            } else if (key == "tempRating") {
                AnimeNode::tempRating = value.toInt(0, 10);
            } else if (key == "tempVoteCount") {
                AnimeNode::tempVoteCount = value.toInt(0, 10);
            } else if (key == "animeID") {
                AnimeNode::id = value.toInt(0, 10);
                GenericTree::setInt(AnimeNode::id);
            } else if (key == "relations") {
                QString relations = value;
                if (relations != "") {
                    AnimeNode::relationsPicturePath = AnimeNode::anidbPath + name + "-relations.png";
                }
                AnimeNode::relations = QValueList<int>();
                QStringList relationList = QStringList::split("'", relations);

                for (QStringList::Iterator it = relationList.begin(); it != relationList.end(); ++it) {
                    QString relation =  *it;
                    AnimeNode::relations.append(relation.toInt());
                }

            } else if (key == "picName") {
                QString picture = value;
                QString extension = picture.right(picture.length() - picture.findRev('.'));

                AnimeNode::picturePath = AnimeNode::anidbPath + name + extension;
            } else if (key == "categoryList") {
                AnimeNode::categoryList = QStringList::split(",", value);
            }
            //TODO check if all values are set
        }

        file.close();


		AnimeNode::overallRating = 1/100.*(AnimeNode::rating*AnimeNode::voteCount+AnimeNode::tempRating*AnimeNode::tempVoteCount)/(AnimeNode::voteCount+AnimeNode::tempVoteCount);
		GenericTree::setAttribute(0, (int)(-100*overallRating));
    } else {
        VERBOSE(VB_IMPORTANT, "Cannot open file " + anidbFile);
    }
}

void AnimeNode::addEpisodes(QString path) {
	QPtrList<EpisodeNode> episodes = listEpisodes(path, getName());
	for (uint j=0; j < episodes.count(); j++) {
		GenericTree::addNode(episodes.at(j));
	}
}

void AnimeNode::readSeenFile() {
    AnimeNode::watched = QStringList();

    QString seenPath = AnimeNode::anidbPath+"seen.txt";
    QFile seenFile( seenPath );

    if ( seenFile.open( IO_ReadOnly ) ) {
		QTextStream stream( &seenFile );

        while ( !stream.atEnd() ) {
			QString line = stream.readLine();
			AnimeNode::watched.append(line);
        }
		seenFile.close();
	} else {
        VERBOSE(VB_FILE, "Cannot open file " + seenPath);
	}

	AnimeNode::watched.sort();
}


QPtrList<EpisodeNode> AnimeNode::listEpisodes(QString path, QString anime) {
    QDir dir(path+"/"+anime);
    dir.setSorting(QDir::Name);
    dir.setFilter(QDir::Files);

    QStringList episodeNames = dir.entryList();
    QPtrList<EpisodeNode> episodes = QPtrList<EpisodeNode>();

    for (uint i=0;i<episodeNames.count();i++) {
        QString name = episodeNames[i];
        episodes.append(new EpisodeNode(anime, name));
    }

    return episodes;
}

bool AnimeNode::isWatched() {
	if (GenericTree::childCount() == 0) {
		VERBOSE(VB_IMPORTANT, "Calling AnimeNode::isWatched on an anime w/o episodes added.");
		return false;
	}
	EpisodeNode* episode = dynamic_cast<EpisodeNode*>(GenericTree::getChildAt(GenericTree::childCount()-1));
	QString number = episode->getNumber();

	return AnimeNode::watched.findIndex(number) != -1;
}

void AnimeNode::addWatched(EpisodeNode* episode) {
	if (AnimeNode::watched.findIndex(episode->getNumber()) == -1) {
		AnimeNode::watched.append(episode->getNumber());


		QString seenPath = AnimeNode::anidbPath+"seen.txt";
		QFile seenFile( seenPath );
		if ( seenFile.open( IO_WriteOnly | IO_Append ) ) {
			QTextStream stream( &seenFile );
			stream << episode->getNumber();
			stream << '\n';

			seenFile.close();
		} else {
			VERBOSE(VB_FILE, "Cannot open file " + seenPath);
		}

		AnimeNode::watched.sort();
	}
}

EpisodeNode* AnimeNode::getEpisodeToWatch() {
	if (GenericTree::childCount() == 0) {
		VERBOSE(VB_IMPORTANT, "Calling AnimeNode::getEpisodeToWatch on an anime w/o episodes added.");
		return NULL;
	}

	if (watched.size() == 0) {
		return (EpisodeNode*)GenericTree::getChildAt(0);
	}


	EpisodeNode* lastWatchedEpisode = NULL;
	EpisodeNode* nextToWatchEpisode = NULL;

	QString lastWatched = *AnimeNode::watched.at(watched.size()-1);

	for (QPtrList<GenericTree>::Iterator iter = GenericTree::getAllChildren()->begin(); iter != GenericTree::getAllChildren()->end(); iter++) {
		EpisodeNode* episode = (EpisodeNode*)*iter;
		if (episode->getNumber() == lastWatched) {
			lastWatchedEpisode = episode;
		} else if (nextToWatchEpisode == NULL && episode->getNumber() > lastWatched) {
			nextToWatchEpisode = episode;
		}
	}

	if (nextToWatchEpisode != NULL) {
		return nextToWatchEpisode;
	} else if (lastWatchedEpisode != NULL) {
		return lastWatchedEpisode;
	} else {
		return (EpisodeNode*)GenericTree::getChildAt(0);
	}
}

void AnimeNode::setSelectedChild() {
	/*
	if (GenericTree::childCount() == 0) {
		VERBOSE(VB_IMPORTANT, "Calling AnimeNode::setSelectedChild on an anime w/o episodes added.");
		return;
	}

	if (watched.size() == 0) {
		GenericTree::setSelectedChild(GenericTree::getChildAt(0));
		return;
	}


	EpisodeNode* lastWatchedEpisode = NULL;
	EpisodeNode* nextToWatchEpisode = NULL;

	QString lastWatched = *AnimeNode::watched.at(watched.size()-1);

	for (QPtrList<GenericTree>::Iterator iter = GenericTree::getAllChildren()->begin(); iter != GenericTree::getAllChildren()->end(); iter++) {
		EpisodeNode* episode = (EpisodeNode*)*iter;
		if (episode->getNumber() == lastWatched) {
			lastWatchedEpisode = episode;
		} else if (nextToWatchEpisode == NULL && episode->getNumber() > lastWatched) {
			nextToWatchEpisode = episode;
		}
	}

	if (nextToWatchEpisode != NULL) {
		GenericTree::setSelectedChild(nextToWatchEpisode);
	} else if (lastWatchedEpisode != NULL) {
		GenericTree::setSelectedChild(lastWatchedEpisode);
	} else {
		GenericTree::setSelectedChild(GenericTree::getChildAt(0));
	}
*/
	EpisodeNode* nextEpisode = getEpisodeToWatch();
	GenericTree::setSelectedChild(nextEpisode);

}
