#ifndef ANIMENODE_H_INCLUDED
#define ANIMENODE_H_INCLUDED

#include <mythtv/uitypes.h>
#include <qstring.h>

#include "episodenode.h"

/**
A GenericTree node describing an anime.
GenericTree::getAttribute(0) will be set to -10000*overallrating
GenericTree::getInt will be set to getId.
*/
class AnimeNode : public GenericTree {

public:
	/**
	Reads all available information for the given anime in the given path.
	*/
    AnimeNode(QString path, QString name);
	~AnimeNode();

private:
	QString anidbPath;

    QStringList watched;

    QString name;
    QString englishName;
    QString romanjiName;
    QString description;

    QString picturePath;


    int startYear;
    int endYear;

    int rating;
    int voteCount;

    int tempRating;
    int tempVoteCount;

    float overallRating;

    int id;
    QValueList<int> relations;
    QString relationsPicturePath;

    QStringList categoryList;

public:
    QString getPicturePath() { return picturePath;}
    QString getRelationsPicturePath() { return relationsPicturePath;}
    QStringList getCategories() { return categoryList;}
    /**
    returns true if the anime was completely watched before
    */
    QString getName() { return name;}
    QString getDescription() { return description;}
    QValueList<int> getRelations() { return relations;}
    int getId() { return id;}
    int getStartYear() { return startYear;}
    int getEndYear() { return endYear;}
    /**
    returns the overall rating for this anime.
    the rating is determined by the following formula:
     1/100*(rating*voteCount+tempRating*tempVoteCount)/(voteCount+tempVoteCount)
    */
    float getOverallRating() { return overallRating;}

	/**
	returns true if and only if watched contains the number of the child with the highest id
	(the episode number of the last episode)
	*/
	bool isWatched();
	/**
	Adds the given number of the episode to the list of watched episodes
	*/
	void addWatched(EpisodeNode* episode);
	/**
	Sets this anime's selected child to the episode after the one that was watched last
	(or to the last episode, if the anime isWatched()).
	The "Selected" child is the child which will be selected if the user changes bins in the
	treelist.
	*/
	void setSelectedChild();
	/**
	Returns this anime's child that is the episode after the one that was watched last
	(or the last episode, if the anime isWatched()).
	*/
	EpisodeNode* getEpisodeToWatch();


private:
    /**
    Returns a list of episodes. One node for each episode.
    Searches in path/anime/
    */
    QPtrList<EpisodeNode> listEpisodes(QString path, QString anime);
    /**
    Reads all episodes in path/getName() and adds them as children to this anime.
    */
    void addEpisodes(QString path);
    /**
    Reads information from the anidb file and initialises found fields.
    */
    void readAniDBFile();
    /**
    Reads information from the anidb-seen file and initialises the watched list.
    */
	void readSeenFile();

};


#endif //ANIMENODE_H_INCLUDED
