#include "animerelations.h"

#include <mythtv/mythcontext.h>




AnimeRelations::AnimeRelations(AnimeNode* anime, QPtrList<AnimeNode> animeList, MythMainWindow *parent, QString windowName,
                                    QString themeFilename, const char *name)
           : MythThemedDialog(parent, windowName, themeFilename, name) {

    AnimeRelations::animeList = animeList;

    initTheme(anime);
}


AnimeRelations::~AnimeRelations() {
	delete AnimeRelations::root;
}


void AnimeRelations::initTheme(AnimeNode* anime) {
    AnimeRelations::image = getUIImageType("image");
    if (!AnimeRelations::image) {
        VERBOSE(VB_IMPORTANT, "Cannot find image image");
    } else {
        setImage(anime);
    }



    AnimeRelations::relatedList = getUIManagedTreeListType("related");
    if (!AnimeRelations::relatedList) {
        VERBOSE(VB_IMPORTANT, "Cannot find managedtreelst related");
    } else {
        initTreeList(anime);
    }


    AnimeRelations::title = getUITextType("title");
    if (!AnimeRelations::title) {
        VERBOSE(VB_IMPORTANT, "Cannot find textbox title");
    } else {
        AnimeRelations::title->SetText(anime->getName());
    }
}

void AnimeRelations::setImage(AnimeNode* anime) {
    QImage relationsImg = QImage(anime->getRelationsPicturePath());
    if (!relationsImg.isNull()) {
        QSize size = AnimeRelations::image->GetSize();
        const int targetWidth = size.width();
        const int targetHeight = size.height();

        QImage image = relationsImg.scale(targetWidth, targetHeight, QImage::ScaleMin);
        QPixmap pixmap = image;

        AnimeRelations::image->SetImage(pixmap);
        AnimeRelations::image->refresh();
    } else {
        AnimeRelations::image->ResetImage();
    }

    QWidget::update();
}

void AnimeRelations::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Anime", e, actions);

    for (unsigned int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];

        handled = true;
        if (action == "UP") {
            AnimeRelations::relatedList->moveUp();
        } else if (action == "DOWN") {
            AnimeRelations::relatedList->moveDown();
        } else if (action == "LEFT") {
            AnimeRelations::relatedList->popUp();
        } else if (action == "RIGHT") {
            AnimeRelations::relatedList->pushDown();
        } else if (action == "PAGEUP") {
            AnimeRelations::relatedList->pageUp();
        } else if (action == "PAGEDOWN") {
            AnimeRelations::relatedList->pageDown();
        } else if (action == "SELECT") {
            AnimeRelations::relatedList->select();
        } else {
            handled = false;
            VERBOSE(VB_IMPORTANT, action);
        }
    }

    if (!handled) {
        MythThemedDialog::keyPressEvent(e);
    }
}

void AnimeRelations::initTreeList(AnimeNode* anime) {
    AnimeRelations::root = new GenericTree();

    QValueList<int> relations = anime->getRelations();
    for (QValueList<int>::Iterator it = relations.begin(); it != relations.end(); ++it) {
        int relationId = *it;

        AnimeNode* relation = findAnime(relationId);
        if (relation) {
            root->addNode(getString(relationId, relation), relationId, true);
        } else {
            root->addNode(getString(relationId, relation), relationId, false);
        }
    }

    AnimeRelations::relatedList->assignTreeData(root);

    connect(AnimeRelations::relatedList, SIGNAL(nodeSelected(int, IntVector*)), this, SLOT(handleTreeListSelection(int, IntVector*)));
}

void AnimeRelations::handleTreeListSelection(int nodeInt, IntVector*) {
    done(1);
}


AnimeNode* AnimeRelations::findAnime(int relation) {
    for (QPtrList<AnimeNode>::Iterator it = AnimeRelations::animeList.begin(); it != AnimeRelations::animeList.end(); ++it) {
        AnimeNode* curAnime = *it;
        if (curAnime->getId() == relation) {
            return curAnime;
        }
    }
    return NULL;
}

//TODO write a method to return the animenode, so those can be set as unselectable
QString AnimeRelations::getString(int relation, AnimeNode* anime) {
    if (!anime) {
        return QString::number(relation);
    }

    QString year = "";
    if (anime->getStartYear() != 0 || anime->getEndYear() != 0) {
        year = " - (";
        if (anime->getStartYear() != 0) {
            year += QString::number(anime->getStartYear());
        }
        if (anime->getStartYear() != 0 && anime->getEndYear() != 0) {
            year += "-";
        }
        if (anime->getEndYear() != 0) {
            year += QString::number(anime->getEndYear());
        }

        year += ")";
    }

    return QString::number(relation) + " - " + anime->getName() + year;
}
