#ifndef ANIMERELATIONS_H_INCLUDED
#define ANIMERELATIONS_H_INCLUDED

#include <mythtv/uitypes.h>
//#include <mythtv/uilistbtntype.h>
//#include <mythtv/xmlparse.h>
//#include <mythtv/mythdialogs.h>

#include "animenode.h"

class AnimeRelations : public MythThemedDialog {
    Q_OBJECT
    typedef QValueVector<int> IntVector;

public:
    /**
    additional arguments
    anime: the anime for which the Relationsrmation will be shown
    */
    AnimeRelations(AnimeNode* anime, QPtrList<AnimeNode> animeList, MythMainWindow *parent, QString windowName,
           QString themeFilename, const char *name = 0);
    ~AnimeRelations();

    /**
    exec will return 1 if the user selected a new anime. 0 if the user canceled.
    */

private:
    UITextType* title;
    UIImageType* image;
    UIManagedTreeListType* relatedList;

    QPtrList<AnimeNode> animeList;
    GenericTree* root;


public:
    /**
    returns the selected related anime. should only be called, if exec indicates a successful selection.
    */
    int getSelectedAnime() { return relatedList->getCurrentNode()->getInt();}

protected:
    /**
    Called automatically on keypress.
    */
    void keyPressEvent(QKeyEvent *e);


private:
    /**
    initializes the theme (gets all UITypes from the xml files), sets the texts and images.
    */
    void initTheme(AnimeNode* anime);
    /**
    populates the managedtreelist with related anime
    */
    void initTreeList(AnimeNode* anime);
    /**
    sets the (relations)image from the given anime
    */
    void setImage(AnimeNode* anime);
    /**
    Returns a QString representing the given anime id. If anime is NULL, only the id will be returned as string.
    */
    QString getString(int animeid, AnimeNode* anime);
    /**
    searches for the given anime id in the list of known anime. returns NULL if nothing is found.
    */
    AnimeNode* findAnime(int animeid);





public slots:
    /**
    To be called when the user points to an item in the treelist
    */
    void handleTreeListEntry(int nodeInt, IntVector*);
    /**
    To be called when the user selects an item in the treelist (see initTheme()).
    */
    void handleTreeListSelection(int nodeInt, IntVector*);
};

#endif // ANIMERELATIONS_H_INCLUDED
