/* QT includes */
#include <qnamespace.h>
#include <qstringlist.h>
#include <qapplication.h>
#include <qbuttongroup.h>

/* MythTV includes */
#include <mythtv/mythcontext.h>
#include <mythtv/mythdialogs.h>

#include <qdir.h>
#include <qdatetime.h>


#include "mythanime.h"
#include "settings.h"
#include "animefilter.h"
#include "animeinfo.h"
#include "animerelations.h"
#include "playerconfig.h"
#include "episodenode.h"


MythAnime::MythAnime(MythMainWindow *parent, QString windowName,
                           QString themeFilename, const char *name)
        : MythThemedDialog(parent, windowName, themeFilename, name) {
	MythAnime::SETTING_PATH = "MythAnime.path";
    MythAnime::SETTING_LASTWATCHED = "MythAnime.last_watched";
    MythAnime::SETTING_HOSTNAME = "generichost";
    MythAnime::SETTING_AID = "MythAnime.aid";
    MythAnime::SETTING_SID = "MythAnime.sid";
	MythAnime::SETTING_PLAYER = "MythAnime.player";
	MythAnime::SETTING_PLAYER_COMMAND = "MythAnime.player.command";

    MythAnime::PATH = gContext->GetSettingOnHost(MythAnime::SETTING_PATH, MythAnime::SETTING_HOSTNAME);
    if (MythAnime::PATH == "") {
    	MythAnime::PATH = "/mnt/shared/anime/";
		gContext->SaveSettingOnHost(MythAnime::SETTING_PATH, MythAnime::PATH, MythAnime::SETTING_HOSTNAME);
    }

    MythAnime::currentTree = NULL;

    initTheme();

    MythAnime::category = "";
    MythAnime::onlyUnwatched = FALSE;
    fillTreeList();


}


MythAnime::~MythAnime() {
	if(currentTree) {
		delete currentTree;
	}
}



void MythAnime::fillTreeList() {
    GenericTree* tree = new GenericTree("Anime", 0, false);

	AnimeNode* lastWatched = NULL;
	int lastWatchedId = gContext->GetNumSettingOnHost(MythAnime::SETTING_LASTWATCHED, SETTING_HOSTNAME, -1);


	QPtrList<AnimeNode> animes = listAnime(true);
    for (uint i=0; i < animes.count(); i++) {
        AnimeNode* anime = animes.at(i);
        tree->addNode(anime);

		if (anime->getId() == lastWatchedId) {
        	lastWatched = anime;
        }
    }


    MythAnime::treeList->assignTreeData(tree);
    if (lastWatched != NULL) {
		QValueList<int> path;
		path << 0;
		path << lastWatchedId;
		path << lastWatched->getEpisodeToWatch()->getInt();
		MythAnime::treeList->tryToSetActive(path);
    }
    MythAnime::treeList->refresh();
    if(currentTree) {
		delete currentTree;
	}
    MythAnime::currentTree = tree;

    handleTreeListUpdate();

    sortTreeList();
}

void MythAnime::sortTreeList() {
	if (MythAnime::sortText->GetText() == "name") {
		treeList->scrambleParents(false);
		MythAnime::treeList->sortTreeByString();
	} else {
		//FIXME implement independent from EpisodeNode::getAttribute(0)
		MythAnime::currentTree->sortByAttributeThenByString(0);
		MythAnime::treeList->scrambleParents(true);
		MythAnime::treeList->setTreeOrdering(0);
		MythAnime::treeList->setVisualOrdering(0);
	}

	MythAnime::treeList->refresh();
}

void MythAnime::initTheme() {
    MythAnime::treeList = getUIManagedTreeListType("treelist");
    if (!MythAnime::treeList) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find managedtreelist treelist");
    } else {
     	MythAnime::treeList->showWholeTree(true);
		MythAnime::treeList->colorSelectables(true);

		connect(MythAnime::treeList, SIGNAL(nodeSelected(int, IntVector*)), this, SLOT(handleTreeListSelection(int, IntVector*)));
		connect(MythAnime::treeList, SIGNAL(nodeEntered(int, IntVector*)), this, SLOT(handleTreeListEntry(int, IntVector*)));
    }

    MythAnime::animetitleText = getUITextType("animetitle");
    if (!MythAnime::animetitleText){
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find textarea animetitle");
    }

    MythAnime::episodetitleText = getUITextType("episodetitle");
    if (!MythAnime::episodetitleText) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find textarea episodetitle");
    }

    MythAnime::episodenumberText = getUITextType("episodenumber");
    if (!MythAnime::episodenumberText) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find textarea episodenumber");
    }

    MythAnime::audio = gContext->GetNumSettingOnHost(MythAnime::SETTING_AID, SETTING_HOSTNAME, 1);
    MythAnime::audioText = getUITextType("audio_val");
    if (!MythAnime::audioText) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find textarea audio_val");
    } else {
    	MythAnime::audioText->SetText(QString::number(MythAnime::audio));
    }

    MythAnime::subtitle = gContext->GetNumSettingOnHost(MythAnime::SETTING_SID, SETTING_HOSTNAME, 0);
    MythAnime::subtitleText = getUITextType("subtitle_val");
    if (!MythAnime::subtitleText) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find textarea subtitle_val");
    } else {
    	MythAnime::subtitleText->SetText(QString::number(MythAnime::subtitle));
    }

    MythAnime::playerText = getUITextType("player_val");
    if (!MythAnime::playerText) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find textarea player_val");
    } else {
        QString player = gContext->GetSettingOnHost(MythAnime::SETTING_PLAYER, MythAnime::SETTING_HOSTNAME);
        QString command = gContext->GetSettingOnHost(MythAnime::SETTING_PLAYER_COMMAND, MythAnime::SETTING_HOSTNAME);
        if (player == "" || command == "") {
			PlayerConfig playerConfig(gContext->GetMainWindow(), "anime-playerconfig", "anime-");
			playerConfig.checkDB();
			player = gContext->GetSettingOnHost(MythAnime::SETTING_PLAYER, MythAnime::SETTING_HOSTNAME);
        }
		MythAnime::playerText->SetText(player);
    }

    MythAnime::sortText = getUITextType("sort_val");
    if (!MythAnime::sortText) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find textarea sort_val");
    }

    MythAnime::animeImage = getUIImageType("image");
    if (!MythAnime::animeImage) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find image image");
    }

    MythAnime::relationsImage = getUIImageType("relations");
    if (!MythAnime::relationsImage) {
		VERBOSE(VB_IMPORTANT, "anime: Couldn't find image relations");
    }



}


void MythAnime::handleTreeListSelection(int nodeInt, IntVector*) {
    GenericTree* node = treeList->getCurrentNode();
    EpisodeNode* episodeNode = dynamic_cast<EpisodeNode*>(node);

    if (episodeNode) {
        startVideo(episodeNode);
    }
}

void MythAnime::handleTreeListEntry(int nodeInt, IntVector*) {
    handleTreeListUpdate();
}

void MythAnime::handleTreeListUpdate() {
    GenericTree* node = treeList->getCurrentNode();

    EpisodeNode* episodeNode = dynamic_cast<EpisodeNode*>(node);
    AnimeNode* animeNode = dynamic_cast<AnimeNode*>(node);

    if (animeNode) {
        MythAnime::animetitleText->SetText(node->getString());

        //TODO set last played episode instead of this
        MythAnime::episodetitleText->SetText("");
        MythAnime::episodenumberText->SetText("");

        setImage(animeNode);


        animeNode->setSelectedChild();
    }
    if (episodeNode) {
        animeNode = dynamic_cast<AnimeNode*>(episodeNode->getParent());
        if (animeNode->getString() != MythAnime::animetitleText->GetText()) {
            MythAnime::animetitleText->SetText(animeNode->getString());
            setImage(animeNode);
        }

        MythAnime::episodetitleText->SetText(episodeNode->getTitle());
        MythAnime::episodenumberText->SetText(episodeNode->getNumber());
    }

    MythAnime::selectedAnime = animeNode;
}

void MythAnime::setImage(AnimeNode* anime) {
    QImage animeImg = QImage(anime->getPicturePath());
    if (!animeImg.isNull()) {
        QSize size = MythAnime::animeImage->GetSize();
        const int targetWidth = size.width();
        const int targetHeight = size.height();

        QImage image = animeImg.scale(targetWidth, targetHeight, QImage::ScaleMin);
        QPixmap pixmap = image;

        animeImage->SetImage(pixmap);
        /*
        a simple refresh is not enough if the image size changes!
        do not forget to call update on "this" later.
        */
        MythAnime::animeImage->refresh();
    } else {
        MythAnime::animeImage->ResetImage();
    }


    /*
    simple approach (does not keep aspect ratio)
    animeImage->SetImage(anime->getPicturePath());
    animeImage->LoadImage();
    animeImage->refresh();
    */


    QImage relationsImg = QImage(anime->getRelationsPicturePath());
    if (!relationsImg.isNull()) {
        QSize size = MythAnime::relationsImage->GetSize();
        const int targetWidth = size.width();
        const int targetHeight = size.height();

        QImage image = relationsImg.scale(targetWidth, targetHeight, QImage::ScaleMin);
        QPixmap pixmap = image;

        MythAnime::relationsImage->SetImage(pixmap);
        MythAnime::relationsImage->refresh();
    } else {
        MythAnime::relationsImage->ResetImage();
    }

    QWidget::update();

}

void MythAnime::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Anime", e, actions);

    for (unsigned int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];

        handled = true;

        if (action == "UP") {
            treeList->moveUp();
        } else if (action == "DOWN") {
            treeList->moveDown();
        } else if (action == "LEFT") {
            treeList->popUp();
        } else if (action == "RIGHT") {
            treeList->pushDown();
        } else if (action == "PAGEUP") {
            treeList->pageUp();
        } else if (action == "PAGEDOWN") {
            treeList->pageDown();
        } else if (action == "SELECT") {
            treeList->select();
        } else if (action == "1") {
            audio = (audio+1) % 3;
            QString str = QString::number(audio);
            audioText->SetText(str);

       		gContext->SaveSettingOnHost(MythAnime::SETTING_AID, str, MythAnime::SETTING_HOSTNAME);
       } else if (action == "2") {
            subtitle = (subtitle+1) % 3;
            QString str = QString::number(subtitle);
            subtitleText->SetText(str);

            gContext->SaveSettingOnHost(SETTING_SID, str, SETTING_HOSTNAME);
        } else if (action == "3") {
            PlayerConfig playerConfig(gContext->GetMainWindow(), "anime-playerconfig", "anime-");
            playerConfig.exec();

            QString player = gContext->GetSettingOnHost(MythAnime::SETTING_PLAYER, MythAnime::SETTING_HOSTNAME);
            MythAnime::playerText->SetText(player);

        } else if (action == "4") { //TODO find proper key
            AnimeRelations relations(MythAnime::selectedAnime, listAnimeAutoDelete(false), gContext->GetMainWindow(), "anime-relations", "anime-");
            int retVal = relations.exec();
            if (retVal == 1) {
                //selection successful (done(1) called) - select new anime
                QValueList<int> path = QValueList<int>();
                path += 0;
                path += relations.getSelectedAnime();

                MythAnime::treeList->tryToSetActive(path);
                MythAnime::treeList->refresh();
                update();
                //note: this does not actually trigger any events!
                handleTreeListUpdate();
            }
            //else user aborted - do nothing
        } else if (action == "5") {
        	QString text = MythAnime::sortText->GetText();
			if (text == "name") {
				MythAnime::sortText->SetText("rating");
			} else {
				MythAnime::sortText->SetText("name");
			}
			sortTreeList();
		} else if (action == "6") {
            AnimeSettings settings(gContext->GetMainWindow(), "anime-settings", "anime-");
            settings.exec();

            QString newPath = gContext->GetSettingOnHost(MythAnime::SETTING_PATH, MythAnime::SETTING_HOSTNAME);
            if (MythAnime::PATH != newPath) {
            	MythAnime::PATH = newPath;
            	fillTreeList();
            }
        } else if (action == "MENU") {
            AnimeFilter filter(listAnimeAutoDelete(false), MythAnime::category, MythAnime::onlyUnwatched, gContext->GetMainWindow(), "anime-filter", "anime-");
            int retVal = filter.exec();
            if (retVal != 1) {
            	//user aborted
            	continue;
            }

            //get values from the filter window and update the tree if necessary
            QString newCategory = filter.getCategory();

            bool newOnlyUnwatched = filter.getOnlyUnwatched();

            bool changed = false;
            if (MythAnime::category != newCategory || MythAnime::onlyUnwatched != newOnlyUnwatched) {
                changed = true;
            }
            MythAnime::category  = newCategory;
            MythAnime::onlyUnwatched = newOnlyUnwatched;

            if (changed) {
                fillTreeList();
            }

            /*
            this would create a small popup, we want fullscreen instead
            MythPopupBox* menuPopup = new MythPopupBox(gContext->GetMainWindow(), "anime-filter");
            menuPopup->ShowPopup(this, SLOT(onclose()));
            */
        } else if (action == "INFO") {
            AnimeInfo info(MythAnime::selectedAnime, gContext->GetMainWindow(), "anime-info", "anime-");
            info.exec();
        } else {
            handled = false;
            VERBOSE(VB_IMPORTANT, action);
        }
    }

    if (!handled) {
        MythThemedDialog::keyPressEvent(e);
    }
}

void MythAnime::startVideo(EpisodeNode* episode) {
	QString cmd = gContext->GetSettingOnHost(MythAnime::SETTING_PLAYER_COMMAND, MythAnime::SETTING_HOSTNAME);
    cmd.replace("$AID", QString::number(audio));
    cmd.replace("$SID", QString::number(subtitle));
    QString filename = MythAnime::PATH+"/"+episode->getAnime() + "/" + episode->getLongName();
    filename.replace("\"", "\\\"");
    cmd.replace("$FILE", filename);
    //note: using myth_system instead of system will cause mythtv to stop listening on lirc input.

    QTime time= QTime::currentTime();
    time.start();
    myth_system(cmd);

    int elapsedSeconds = time.elapsed()/1000;
	if (elapsedSeconds > 30) {
		AnimeNode* anime = dynamic_cast<AnimeNode*>(episode->getParent());
		anime->addWatched(episode);

		gContext->SaveSettingOnHost(SETTING_LASTWATCHED, QString::number(anime->getId()), SETTING_HOSTNAME);

		//also select the next episode for playback
		if (anime->isWatched()) {
			fillTreeList();
		} else {
			QValueList<int> path;
			path << 0;
			path << anime->getInt();
			path << anime->getEpisodeToWatch()->getInt();
			MythAnime::treeList->tryToSetActive(path);
			MythAnime::treeList->refresh();
		}
	} else {
		//discard, the user only zapped (for less than 30s)
		//maybe this should be configurable?
	}


}

QPtrList<AnimeNode> MythAnime::listAnimeAutoDelete(bool filter) {
	QPtrList<AnimeNode> retVal = listAnime(filter);
	retVal.setAutoDelete(true);
	return retVal;
}

QPtrList<AnimeNode> MythAnime::listAnime(bool filter) {
    QDir dir(MythAnime::PATH);
    dir.setSorting(QDir::Name);
    dir.setFilter(QDir::Dirs);

    QStringList animeNames = dir.entryList();
    animeNames.remove(".");
    animeNames.remove("..");

    QPtrList<AnimeNode> animes = QPtrList<AnimeNode>();

    for (uint i=0;i<animeNames.count();i++) {
        QString name = animeNames[i];

    	QFile file(MythAnime::PATH+"/"+name+"/anidb/"+name+".txt");
    	if (file.exists()) {
			AnimeNode* node = new AnimeNode(MythAnime::PATH, name);

			if (!filter || !isFiltered(node)) {
				animes.append(node);
			} else {
				delete node;
			}
    	}
    }

    return animes;
}

bool MythAnime::isFiltered(AnimeNode* anime) {
    if (MythAnime::category != "" && anime->getCategories().findIndex(category) == -1) {
        return true;
    }

    if (MythAnime::onlyUnwatched && anime->isWatched()) {
        return true;
    }

    return false;
}


