#ifndef MYTHANIME_H_INCLUDED
#define MYTHANIME_H_INCLUDED

#include <mythtv/uitypes.h>
#include <mythtv/uilistbtntype.h>
#include <mythtv/xmlparse.h>
#include <mythtv/mythdialogs.h>

#include "animenode.h"

#include <unistd.h>


/**
The main plugin window.
*/
class MythAnime : public MythThemedDialog
{
    Q_OBJECT

private:
    UIManagedTreeListType *treeList;
    GenericTree* currentTree;
    UITextType *animetitleText;
    UITextType *episodetitleText;
    UITextType *episodenumberText;
    UITextType *audioText;
    int audio;
    UITextType *subtitleText;
    int subtitle;
    UITextType *playerText;
    UITextType *sortText;
    UIImageType *animeImage;
    UIImageType *relationsImage;

    QString SETTING_HOSTNAME;
    QString PATH;
    QString SETTING_PATH;
    QString SETTING_LASTWATCHED;
    QString SETTING_AID;
    QString SETTING_SID;
    QString SETTING_PLAYER;
    QString SETTING_PLAYER_COMMAND;

    QString category;
    bool onlyUnwatched;
    AnimeNode* selectedAnime;

public:
    typedef QValueVector<int> IntVector;
    /**
    Constructor with args necessary for the super class.
    */
    MythAnime(MythMainWindow *parent, QString windowName,
           QString themeFilename, const char *name = 0);
    ~MythAnime();

protected:
    /**
    Called automatically on keypress.
    */
    void keyPressEvent(QKeyEvent *e);

private:
    /**
    Initializes the theme (gets the ui elements for each item from the xml file for this window.
    Connects the treelist's signals.
    */
    void initTheme();
    /**
    Fills the managedtreelist with values (AnimeNodes and EpisodeNodes) for each anime and each episode.
    each anime gets its animeid as int-value.
    the root's getInt() is 0.
    */
    void fillTreeList();
    /**
    Sorts the treelist according to the sort-order in sortText (sortText->GetText() must be one of 'name' and 'rating')
    note: will only work if attribute(0) of each node of the GenericTree is set!
    */
    void sortTreeList();
    /**
    Starts the given episode with the currently selected player (see playerText).
    - marks the current episode as watched (if played for more than 30s).
    - remembers the anime as the last played one (if played for more than 30s).
    - selects the next episode
    */
    void startVideo(EpisodeNode*);
    /**
    Sets the images (title/relations) for the given anime.
    */
    void setImage(AnimeNode* anime);
    /**
    Returns a list of anime. One node for each anime.
    Searches in PATH.
    if filter is true, only anime matching the active filter will be returned
    */
    QPtrList<AnimeNode> listAnime(bool filter);
	/**
    Returns a list of anime.
    Same result as listAnime, but also calls setAutoDelete(true) on the QPtrList.
    */
    QPtrList<AnimeNode> listAnimeAutoDelete(bool filter);
    /**
    Returns true if the anime is filtered (category not in list of categories or onlyWatched=true, but anime was watched already)
    */
    bool isFiltered(AnimeNode* anime);
    /**
    Shows the appropriate information for the selected anime or episode.
    */
    void handleTreeListUpdate();

public slots:
    /**
    To be called when the user points to an item in the treelist (see initTheme()).
    Shows the appropriate information for the Node (Anime or Episode).
    */
    void handleTreeListEntry(int nodeInt, IntVector*);
    /**
    To be called when the user selects an item in the treelist (see initTheme()).
    Starts the media player.
    */
    void handleTreeListSelection(int nodeInt, IntVector*);
};

#endif // MYTHANIME_H_INCLUDED
