#include "playerconfig.h"

#include <mythtv/mythcontext.h>
#include <mythtv/mythdbcon.h>



PlayerConfig::PlayerConfig(MythMainWindow *parent, QString windowName,
                                    QString themeFilename, const char *name)
           : MythThemedDialog(parent, windowName, themeFilename, name) {
	PlayerConfig::SETTING_DBVERSION = "MythAnime.dbversion";
	PlayerConfig::SETTING_HOSTNAME = "generichost";
	PlayerConfig::SETTING_PLAYER = "MythAnime.player";
	PlayerConfig::SETTING_PLAYER_COMMAND = "MythAnime.player.command";

	PlayerConfig::selectedPlayer = NULL;
	PlayerConfig::root = NULL;

	PlayerConfig::editor = NULL;
	PlayerConfig::menuPopup = NULL;

    initTheme();
}


PlayerConfig::~PlayerConfig() {
	delete PlayerConfig::root;
}


void PlayerConfig::initTheme() {
    PlayerConfig::playerList = getUIManagedTreeListType("players");
    if (!PlayerConfig::playerList) {
        VERBOSE(VB_IMPORTANT, "Cannot find managedtreelist players");
    } else {
     	PlayerConfig::playerList->showWholeTree(true);
		PlayerConfig::playerList->colorSelectables(true);

		connect(PlayerConfig::playerList, SIGNAL(nodeSelected(int, IntVector*)), this, SLOT(handleTreeListSelection(int, IntVector*)));
		connect(PlayerConfig::playerList, SIGNAL(nodeEntered(int, IntVector*)), this, SLOT(handleTreeListEntry(int, IntVector*)));

		initTreeFromSql();
     }
}

void PlayerConfig::initTreeFromSql(QString player) {
	checkDB();

	if (PlayerConfig::root) {
		delete PlayerConfig::root;
	}

	QString oldPlayer;
	if (player == "") {
		oldPlayer = gContext->GetSettingOnHost(PlayerConfig::SETTING_PLAYER, PlayerConfig::SETTING_HOSTNAME);
	} else {
		oldPlayer = player;
	}

	MSqlQuery query(MSqlQuery::InitCon());
	query.prepare("SELECT name, command FROM anime_players ORDER BY name");


	PlayerConfig::root = new GenericTree();

	if (query.exec() && query.isActive()) {
		while (query.next()) {
			QString name = query.value(0).toString();
			QString command = query.value(1).toString();



			if (name == oldPlayer) {
				GenericTree* node = root->addNode(name, 1, true);
				node->addNode(command, 1, true);

				PlayerConfig::selectedPlayer = node;
			} else {
				GenericTree* node = root->addNode(name, 0, true);
				node->addNode(command, 0, true);
			}
		}
	} else {
        MythContext::DBError("player selection", query);
	}

	PlayerConfig::playerList->assignTreeData(PlayerConfig::root);
	QValueList<int> route;
	route << 0 << 1 << 1;
	PlayerConfig::playerList->tryToSetActive(route);
	PlayerConfig::playerList->refresh();
}

void PlayerConfig::checkDB() {
	QString dbver = gContext->GetSettingOnHost(PlayerConfig::SETTING_DBVERSION, PlayerConfig::SETTING_HOSTNAME);
	QString player = gContext->GetSettingOnHost(PlayerConfig::SETTING_PLAYER, PlayerConfig::SETTING_HOSTNAME);
	QString playerCmd = gContext->GetSettingOnHost(PlayerConfig::SETTING_PLAYER_COMMAND, PlayerConfig::SETTING_HOSTNAME);
	if (dbver == "" || player == "" || playerCmd == "") {
		MSqlQuery query(MSqlQuery::InitCon());
		query.exec("CREATE TABLE IF NOT EXISTS anime_players (name VARCHAR(128) NOT NULL PRIMARY KEY, command VARCHAR(512) NOT NULL)");
		query.exec("INSERT INTO anime_players VALUES('mplayer', 'mplayer -aid $AID -sid $SID \"$FILE\"')");
		query.exec("INSERT INTO anime_players VALUES('mplayer-no[as]id', 'mplayer \"$FILE\"')");
		query.exec("INSERT INTO anime_players VALUES('xine', 'xine -a $AID -s $SID \"$FILE\"')");

		gContext->SaveSettingOnHost(PlayerConfig::SETTING_PLAYER, "mplayer", PlayerConfig::SETTING_HOSTNAME);
		gContext->SaveSettingOnHost(PlayerConfig::SETTING_PLAYER_COMMAND, "mplayer -aid $AID -sid $SID \"$FILE\"", PlayerConfig::SETTING_HOSTNAME);

		gContext->SaveSettingOnHost(PlayerConfig::SETTING_DBVERSION, "1", PlayerConfig::SETTING_HOSTNAME);
	}
}

void PlayerConfig::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Anime", e, actions);

    for (unsigned int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];

        handled = true;
        if (action == "UP") {
            PlayerConfig::playerList->moveUp();
        } else if (action == "DOWN") {
            PlayerConfig::playerList->moveDown();
        } else if (action == "LEFT") {
            PlayerConfig::playerList->popUp();
        } else if (action == "RIGHT") {
            PlayerConfig::playerList->pushDown();
        } else if (action == "PAGEUP") {
            PlayerConfig::playerList->pageUp();
        } else if (action == "PAGEDOWN") {
            PlayerConfig::playerList->pageDown();
        } else if (action == "SELECT") {
            PlayerConfig::playerList->select();
        } else if (action == "1") {
			makePopup("Player Name", SLOT(addEntry()), SLOT(cancel()));
        } else if (action == "2") {
        	if (root->childCount() > 1) {
				makePopup("Really Remove?", SLOT(removeEntry()), SLOT(cancel()), false);
        	}
        } else if (action == "3") {
        	QString command = PlayerConfig::selectedPlayer->getChildAt(0)->getString();
        	makePopup("Player command", SLOT(editEntry()), SLOT(cancel()), true, command);
		} else {
            handled = false;
            VERBOSE(VB_IMPORTANT, action);
        }
    }

    if (!handled) {
        MythThemedDialog::keyPressEvent(e);
    }
}

//FIXME wtf?!
void PlayerConfig::makePopup(QString label, const char *okSlot, const char *cancelSlot, QString editText) {
	makePopup(label, okSlot, cancelSlot, true, editText);
}

void PlayerConfig::makePopup(QString label, const char *okSlot, const char *cancelSlot, bool createEditor, QString editText) {
	PlayerConfig::menuPopup = new MythPopupBox(gContext->GetMainWindow(), "player popup");

	PlayerConfig::menuPopup->addLabel(label);

	if (createEditor) {
		PlayerConfig::editor = new MythRemoteLineEdit(PlayerConfig::menuPopup, "huh?");
		PlayerConfig::editor->setText(editText);
		PlayerConfig::menuPopup->addWidget(PlayerConfig::editor);
	}

	PlayerConfig::menuPopup->addButton("OK", this, okSlot);
	PlayerConfig::menuPopup->addButton("CANCEL", this, cancelSlot);
	PlayerConfig::menuPopup->ShowPopup(this, SLOT(cancel()));
}

void PlayerConfig::addEntry(){
	//note delete (in close) does actually destroy the QString! make a copy.
	QString name = QString(PlayerConfig::editor->text());
	//TODO check if name is unused
	if (name != "") {
		PlayerConfig::addEntryName = name;
		cancel();
		makePopup("Command", SLOT(addedEntry()), SLOT(cancel()));
	}
}

void PlayerConfig::addedEntry(){
	QString name = PlayerConfig::addEntryName;
	QString command = QString(PlayerConfig::editor->text());
	if (command != "") {
		//TODO add to db: name, command
		//make active
		//refresh treelist


		MSqlQuery query(MSqlQuery::InitCon());
		query.prepare("INSERT INTO anime_players VALUES(:NAME, :COMMAND)");
		query.bindValue(":NAME",  name);
		query.bindValue(":COMMAND",  command);

		if (!query.exec() || !query.isActive()) {
			MythContext::DBError("player insert", query);
		}

		initTreeFromSql(name);

		cancel();
	}
}


void PlayerConfig::removeEntry(){
	QString name = PlayerConfig::selectedPlayer->getString();
	//delete from table where name=name


	MSqlQuery query(MSqlQuery::InitCon());
	query.prepare("DELETE FROM anime_players WHERE name=:NAME");
	query.bindValue(":NAME",  name);

	if (!query.exec() || !query.isActive()) {
		MythContext::DBError("player delete", query);
	}

	//if deleted player is the selected player: select another player
	if (name == gContext->GetSettingOnHost(PlayerConfig::SETTING_PLAYER, PlayerConfig::SETTING_HOSTNAME)) {
		gContext->SaveSettingOnHost(PlayerConfig::SETTING_PLAYER, PlayerConfig::root->getChildAt(0)->getString(), PlayerConfig::SETTING_HOSTNAME);
		gContext->SaveSettingOnHost(PlayerConfig::SETTING_PLAYER_COMMAND, PlayerConfig::root->getChildAt(0)->getChildAt(0)->getString(), PlayerConfig::SETTING_HOSTNAME);
	}

	initTreeFromSql();
	cancel();
}

void PlayerConfig::editEntry(){
	//just edit the active entry
	QString command = QString(PlayerConfig::editor->text());
	if (command != "") {
		QString name = PlayerConfig::selectedPlayer->getString();


		MSqlQuery query(MSqlQuery::InitCon());
		query.prepare("UPDATE anime_players SET command=:COMMAND where name=:NAME");
		query.bindValue(":COMMAND",  command);
		query.bindValue(":NAME",  name);


		if (!query.exec() || !query.isActive()) {
			MythContext::DBError("player update", query);
		}


		PlayerConfig::selectedPlayer->getChildAt(0)->setString(command);
		PlayerConfig::playerList->refresh();

		cancel();
	}
}



void PlayerConfig::cancel() {
	if (PlayerConfig::menuPopup) {
		PlayerConfig::menuPopup->hide();
		//note: the popup MUST be deleted, or it will not be hidden properly (it'll still grab all input events)
		delete PlayerConfig::menuPopup;
		PlayerConfig::menuPopup = NULL;

		//note: the editor does get deleted automatically (i think)
		PlayerConfig::editor = NULL;
		/*if (PlayerConfig::editor) {
			delete PlayerConfig::editor;
			PlayerConfig::editor = NULL;
		}*/
	}
}



void PlayerConfig::handleTreeListSelection(int nodeInt, IntVector*) {
	gContext->SaveSettingOnHost(PlayerConfig::SETTING_PLAYER, PlayerConfig::selectedPlayer->getString(), PlayerConfig::SETTING_HOSTNAME);
	gContext->SaveSettingOnHost(PlayerConfig::SETTING_PLAYER_COMMAND, PlayerConfig::selectedPlayer->getChildAt(0)->getString(), PlayerConfig::SETTING_HOSTNAME);
	done(1);
}

void PlayerConfig::handleTreeListEntry(int nodeInt, IntVector*) {
	GenericTree* node = playerList->getCurrentNode();
	if (node->getParent() != PlayerConfig::root) {
		node = node->getParent();
	}

	PlayerConfig::selectedPlayer = node;
}

