#ifndef PLAYERCONFIG_H_INCLUDED
#define PLAYERCONFIG_H_INCLUDED

#include "mythanime.h"

#include <mythtv/uitypes.h>

/**
does not support undoing changes
*/
class PlayerConfig : public MythThemedDialog {
	Q_OBJECT
    typedef QValueVector<int> IntVector;

public:
    /**
    additional arguments
    */
    PlayerConfig(MythMainWindow *parent, QString windowName,
           QString themeFilename, const char *name = 0);
    ~PlayerConfig();
/*
    class Player {
    	public:
			QString name;
			QString command;
    };*/

	/**
	returns the selected player
	*/
    QString getSelectedPlayer() { return selectedPlayer->getString();}
    /**
    checks if a sql table exists and creates one if necessary
    also initializes the player setting
    */
    void checkDB();

private:
	UIManagedTreeListType* playerList;
	GenericTree* root;
	/**
	the currently selected player
	if a command is selected, the player for that command
	*/
	GenericTree* selectedPlayer;

	QString SETTING_DBVERSION;
	QString SETTING_HOSTNAME;
	QString SETTING_PLAYER;
	QString SETTING_PLAYER_COMMAND;


private:
	/**
	initializes the theme (gets the ui elements from mythtv and sets their initial values)
	*/
    void initTheme();
    /**
    retrieves the players from sql and puts them into the playerlist.
    also sets root
    sets the selected entry to the given player (or to the active player)
    */
    void initTreeFromSql(QString player = "");




private:
	MythPopupBox* menuPopup;
	MythRemoteLineEdit* editor;
	QString addEntryName;

	/**
	creates a MythPopupBox (menuPopup) with the given label, a MythRemoteLineEdit (editor),
	an ok and a cancel button. The buttons use the given slots respectively.
	if createEditor is false, no editor will be created.
	*/
	void makePopup(QString label, const char *okSlot, const char *cancelSlot, bool createEditor = true, QString editText = "");

	/**
	same as makePopup(label, okSlot, cancelSlot, true, editText)
	*/
	void makePopup(QString label, const char *okSlot, const char *cancelSlot, QString editText);


public slots:
    /**
	adds a new entry to the db (first popup: name)
    */
	void addEntry();
	/**
	adds a new entry to the db (second popup: command)
    */
	void addedEntry();
    /**
    removes the currently selected entry from the db
    popup: are you sure?
    */
	void removeEntry();
    /**
    edits the current entry
    */
	void editEntry();
    /**
    closes the active popup
	*/
	void cancel();


protected:
    /**
    Called automatically on keypress.
    */
    void keyPressEvent(QKeyEvent *e);

public slots:
    /**
    To be called when the user points to an item in the treelist
    */
    void handleTreeListEntry(int nodeInt, IntVector*);
    /**
    To be called when the user selects an item in the treelist (see initTheme()).
    */
    void handleTreeListSelection(int nodeInt, IntVector*);

};

#endif // PLAYERCONFIG_H_INCLUDED
