#include "settings.h"

#include <mythtv/mythcontext.h>




AnimeSettings::AnimeSettings(MythMainWindow *parent, QString windowName,
                                    QString themeFilename, const char *name)
           : MythThemedDialog(parent, windowName, themeFilename, name) {
	AnimeSettings::SETTING_PATH = "MythAnime.path";
	AnimeSettings::SETTING_ANIDBPATH = "MythAnime.anidbpath";
    AnimeSettings::SETTING_HOSTNAME = "generichost";


    QString anidbPath = gContext->GetSettingOnHost(AnimeSettings::SETTING_ANIDBPATH, AnimeSettings::SETTING_HOSTNAME);
    if (anidbPath == "") {
		gContext->SaveSettingOnHost(AnimeSettings::SETTING_ANIDBPATH, "/opt/anidb/", AnimeSettings::SETTING_HOSTNAME);
    }


    initTheme();
}


AnimeSettings::~AnimeSettings() {
}


void AnimeSettings::initTheme() {
    AnimeSettings::path = getUITextType("path_val");
    if (!AnimeSettings::path) {
        VERBOSE(VB_IMPORTANT, "Cannot find textbox path_val");
    } else {
        AnimeSettings::path->SetText(gContext->GetSettingOnHost(AnimeSettings::SETTING_PATH, AnimeSettings::SETTING_HOSTNAME));
    }

	AnimeSettings::anidbPath = getUITextType("anidbpath_val");
    if (!AnimeSettings::anidbPath) {
        VERBOSE(VB_IMPORTANT, "Cannot find textbox anidbpath_val");
    } else {
        AnimeSettings::anidbPath->SetText(gContext->GetSettingOnHost(AnimeSettings::SETTING_ANIDBPATH, AnimeSettings::SETTING_HOSTNAME));
    }
}



void AnimeSettings::keyPressEvent(QKeyEvent *e) {
    bool handled = false;
    QStringList actions;
    gContext->GetMainWindow()->TranslateKeyPress("Anime", e, actions);

    for (unsigned int i = 0; i < actions.size() && !handled; i++)
    {
        QString action = actions[i];

        handled = true;
        if (action == "1") {
			AnimeSettings::menuPopup = new MythPopupBox(gContext->GetMainWindow(), "paths popup");

			AnimeSettings::menuPopup->addLabel("New Path");

			AnimeSettings::editor = new MythRemoteLineEdit(AnimeSettings::menuPopup, "huh?");
			AnimeSettings::editor->setText(AnimeSettings::path->GetText());
			AnimeSettings::menuPopup->addWidget(AnimeSettings::editor);

			AnimeSettings::menuPopup->addButton("OK", this, SLOT(changePath()));
			AnimeSettings::menuPopup->addButton("CANCEL", this, SLOT(cancel()));
			AnimeSettings::menuPopup->ShowPopup(this, SLOT(cancel()));
		} else if (action == "2") {
			AnimeSettings::menuPopup = new MythPopupBox(gContext->GetMainWindow(), "paths popup");

			AnimeSettings::menuPopup->addLabel("AniDB Parser");

			AnimeSettings::editor = new MythRemoteLineEdit(AnimeSettings::menuPopup, "huh?");
			AnimeSettings::editor->setText(AnimeSettings::anidbPath->GetText());
			AnimeSettings::menuPopup->addWidget(AnimeSettings::editor);

			AnimeSettings::menuPopup->addButton("OK", this, SLOT(changeAnidbPath()));
			AnimeSettings::menuPopup->addButton("CANCEL", this, SLOT(cancel()));
			AnimeSettings::menuPopup->ShowPopup(this, SLOT(cancel()));
        } else if (action == "3") {
        	QString anidbPath = gContext->GetSettingOnHost(AnimeSettings::SETTING_ANIDBPATH, AnimeSettings::SETTING_HOSTNAME);
        	QString path = gContext->GetSettingOnHost(AnimeSettings::SETTING_PATH, AnimeSettings::SETTING_HOSTNAME);
			QString command = "xterm -e 'cd \""+anidbPath+"\"; java -cp .:anidb.jar honk.tools.anidb.AniDB identify \""+path+"\"; read'";
			//VERBOSE(VB_IMPORTANT, command);
			myth_system(command);
		} else if (action == "4") {
        	QString anidbPath = gContext->GetSettingOnHost(AnimeSettings::SETTING_ANIDBPATH, AnimeSettings::SETTING_HOSTNAME);
        	QString path = gContext->GetSettingOnHost(AnimeSettings::SETTING_PATH, AnimeSettings::SETTING_HOSTNAME);
			QString command = "xterm -e 'cd \""+anidbPath+"\"; java -cp .:anidb.jar honk.tools.anidb.AniDB move \""+path+"\"  \""+path+"\"; read'";
			myth_system(command);
        } else {
            handled = false;
            VERBOSE(VB_IMPORTANT, action);
        }
    }

    if (!handled) {
        MythThemedDialog::keyPressEvent(e);
    }
}


void AnimeSettings::changePath() {
	QString path = QString(AnimeSettings::editor->text());
	if (path != "") {
		gContext->SaveSettingOnHost(AnimeSettings::SETTING_PATH, path, AnimeSettings::SETTING_HOSTNAME);
		AnimeSettings::path->SetText(path);
		cancel();
	}
}

void AnimeSettings::changeAnidbPath() {
	QString path = QString(AnimeSettings::editor->text());
	if (path != "") {
		gContext->SaveSettingOnHost(AnimeSettings::SETTING_ANIDBPATH, path, AnimeSettings::SETTING_HOSTNAME);
		AnimeSettings::anidbPath->SetText(path);
		cancel();
	}
}

void AnimeSettings::cancel() {
	if (AnimeSettings::menuPopup) {
		AnimeSettings::menuPopup->hide();
		//note: the popup MUST be deleted, or it will not be hidden properly (it'll still grab all input events)
		delete AnimeSettings::menuPopup;
		AnimeSettings::menuPopup = NULL;

		//note: the editor does get deleted automatically (i think)
		AnimeSettings::editor = NULL;
	}
}
